package GII::Event::PButton;

=head1 NAME

GII::Event::PButton - Perl class encapsulating gii_pbutton_event structures.

=head1 SYNOPSIS

    use GII::Event::PButton;

    $ev = new GII::Event::PButton;

    $button = $ev->button;
    $ev->button($button);

    $button_down = $ev->press;
    $ev->press($button_down);

    $button_repeated = $ev->repeat;
    $ev->repeat($button_repeated);

    $button_up = $ev->release;
    $ev->release($button_up);

=head1 DESCRIPTION

The GII::Event::PButton subclass provides simple methods to access
the extra member and state information of a LibGII gii_pbutton_event 
structure with cannot be accessed via the methods in the class GII::Event.

=cut

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use GII::Event;

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter AutoLoader DynaLoader GII::Event);

=head1 Exported Constants

PBUTTON_NONE
PBUTTON_LEFT
PBUTTON_PRIMARY
PBUTTON_FIRST
PBUTTON_RIGHT
PBUTTON_SECONDARY
PBUTTON_SECOND
PBUTTON_MIDDLE
PBUTTON_TERTIARY
PBUTTON_THIRD

=cut

@EXPORT = qw(
PBUTTON_NONE
PBUTTON_LEFT
PBUTTON_PRIMARY
PBUTTON_FIRST
PBUTTON_RIGHT
PBUTTON_SECONDARY
PBUTTON_SECOND
PBUTTON_MIDDLE
PBUTTON_TERTIARY
PBUTTON_THIRD
);

$VERSION = '0.01';

bootstrap GII::Event::PButton $VERSION;

=head1 Methods

The extra methods and differences between GII::Event methods and the
ones provided in the GII::Event::PButton class are as follows:

=head2

The GII::Event::PButton constructor takes all the same arguments
as the GII::Event constructor, and treats them the same with the
exception that it does not do any automagical subclassing.  All
calls to this constructor will either fail or allocate/reuse
a gii_pbutton_event structure.

Since this event is generated by the environment, this constructor
will be used only very rarely for debugging purposes.

=cut

sub new {
    my($class) = shift();
    my($type) = shift();
    my($this) = {};
    bless $this, $class;
    my($obj) = shift();
    if ($obj) {
        $this->{obj} = $obj;
        $this->set_type($obj, $type) if defined($type);
        $this->set_size($obj, $this->default_size) if defined($type);
        return($this);
    }
    $this->{obj} = $this->_alloc;
    $this->set_size($this->{obj}, $this->default_size);
    $this->set_type($this->{obj}, $type);
    return($this);
}

=head2 button

Without an argument, returns the number of the pointer button
which has changed state.  With an argument, sets said value, 
returning the value which was set.

=cut

sub button {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_button($this->{obj}, shift());
    }
    return $this->get_button($this->{obj});
}

=head2 press

Without an argument, returns 0 if the event is a button release,
1 if the event is a button press, and (were it implemented in LibGII
for mice buttons it would return) 2 if the event is an automatic 
button repeat.  If the type is invalid for this event class,
returns undef.

With an argument, sets the event to a button press if the argument
is true in a boolean context, else to a button release, and returns 
the LibGII gii_event_type which was set.

=cut

sub press {
    my($this) = shift();
    if (@_ + 0) {
        if (shift()) {
            $this->set_type($this->{obj}, &evPtrButtonPress);
            return &evPtrButtonPress;
        }
        else {
            $this->set_type($this->{obj}, &evPtrButtonRelease);
            return &evPtrButtonRelease;
        }
    }
    return 1 if $this->get_type($this->{obj}) == &evPtrButtonPress;
    return 0 if $this->get_type($this->{obj}) == &evPtrButtonRelease;
#    return 2 if $this->get_type($this->{obj}) == &evPtrButtonRepeat;
    return undef;
}

=head2 release

Without an argument, returns 1 if the event is a button release,
0 if the event is a button press (or were it implemented in LibGII
for mice buttons, a button repeat.)  If the type is invalid for 
this event class, returns undef.

With an argument, sets the event to a button release if the argument
is true in a boolean context, else to a button press event, 
and returns the LibGII gii_event_type which was set.

=cut

sub release {
    my($this) = shift();
    if (@_ + 0) {
        if (shift()) {
            $this->set_type($this->{obj}, &evPtrButtonRelease);
            return &evPtrButtonRelease;
        }
        else {
            $this->set_type($this->{obj}, &evPtrButtonPress);
            return &evPtrButtonPress;
        }
    }

    return 1 if $this->get_type($this->{obj}) == &evPtrButtonRelease;
    return 0 if $this->get_type($this->{obj}) == &evPtrButtonPress;
#    return 0 if $this->get_type($this->{obj}) == &evPtrButtonRepeat;
    return undef;
}

=head2 repeat

(IMPLEMENTATION NOTE: LibGII currently does not define a button
repeat event, so the below does not really work.)

Without an argument, returns 1 if the event is a button repeat,
0 if the event is a button press or release.  If the type is 
invalid for this event class, returns undef.

With an argument, sets the event to a button repeat if the argument
is true in a boolean context, else to a button release, and returns 
the LibGII gii_event_type which was set.

=cut

sub repeat {
    my($this) = shift();
    if (@_ + 0) {
        if (shift()) {
            $this->set_type($this->{obj}, &evPtrButtonPress);
            return &evPtrButtonPress;
        }
        else {
            $this->set_type($this->{obj}, &evPtrButtonRelease);
            return &evPtrButtonRelease;
        }
    }
#    return 1 if $this->get_type($this->{obj}) == &evPtrButtonRepeat;
    return 0 if $this->get_type($this->{obj}) == &evPtrButtonPress;
    return 0 if $this->get_type($this->{obj}) == &evPtrButtonRelease;
    return undef;
}


1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event(3pm), GII::Event::PMove(3pm), GII(3pm), perl(1).

=cut
